@@ -1,6 +1,7 @@
 #!/bin/sh
 
 verbose="${VERBOSE:-0}"
+readonly verbose
 
 if [ "$verbose" -gt '0' ]
 then
@@ -9,23 +10,20 @@ fi
 
 set -e -f -u
 
-dist_dir="$DIST_DIR"
-go="${GO:-go}"
+dist_dir="${DIST_DIR:?please set DIST_DIR}"
+sudo_cmd="${SUDO:-}"
+readonly dist_dir sudo_cmd
 
-# Set the GOPATH explicitly in case make clean is called from under sudo
-# after a Docker build.
-env PATH="$("$go" env GOPATH)/bin":"$PATH" packr clean
-
-rm -f\
+$sudo_cmd rm -f\
 	./AdGuardHome\
 	./AdGuardHome.exe\
 	./coverage.txt\
 	;
 
-rm -f -r\
+$sudo_cmd rm -f -r\
 	./bin/\
-	./build/\
-	./build2/\
+	./build/static/\
+	./build2/static/\
 	./client/node_modules/\
 	./client2/node_modules/\
 	./data/\
